package outils;
import simu.*;
import outils.*;

/**
 * Bibliothèque contenant des outils pour travailler sur des fonctions 1D ou 2D et résoudre des équations différentiels dans différents cas.
 *
 *
 *
 */


public class Bibliotheque {

/**
 * Méthode d'Euler modifiée.
 *
 * Pour une équation différentielle du type : Y' = F(Y), alors au pas i+1 on a :
 *
 * Y(i+1) = Y(i) + pas/2 * ( F(Y(i)) + F(Y(i+1)) )
 *
 * avec F( Y(i+1) ) ~ F( Y(i) + pas*F(Y(i)) ) , où on a utilisé Euler simple comme estimateur de F(Y(i+1))
 *
 */

    public static double rk2(Fonction f, double y0, double pas)

    {
        return y0 + pas/2.0 * ( f.y(y0) + f.y( y0 + pas*f.y(y0) ) );
    }

/**
 *
 * Runge-Kutta d'ordre 4, pour un équation d'ordre 1.
 * Pour une équation différentielle du type : Y' = F(Y), alors au pas i+1 on a :
 * Y(i+1)=Y(i)+pas*1/6*(F1 + 2F2 + 2F3 + F4)
 *
 * , où les Fi sont différentes estimations de la pente:
 * F1(Yi)
 * F2(Yi+h/2 F1)
 * F3(Yi+h/2 F2)
 * F4(Yi+h F3)
 *
 *
 */


    public static double rk4(Fonction f, double y0, double pas)

    {
        double F1,F2,F3,F4;

        F1=f.y(y0);
        F2=f.y(y0+pas*0.5*F1);
        F3=f.y(y0+pas*0.5*F2);
        F4=f.y(y0+pas*F3);

        return y0 + pas * ( F1 + 2.*F2 + 2.*F3 + F4 ) /6.0;
    }


/**
*
* Runge-Kutta d'ordre 4 à pas variable
* Pour une équation différentielle du type : Y' = F(Y), alors au pas i+1 on a :
* Y(i+1)=Y(i)+pas*1/6*(F1 + 2F2 + 2F3 + F4)
*
* , où les Fi sont différentes estimations de la pente:
* F1(Yi)
* F2(Yi+h/2 F1)
* F3(Yi+h/2 F2)
* F4(Yi+h F3)
*
* Cette méthode prend en plus l'argument 'tol' qui correspond à la tolérance consentie
* dans l'erreur commise entre deux manières d'estimer la fonction au pas i+1
* Ces deux manières sont : estimation en 1 pas de valeur 'pas' de i à i+1 ;
* et estimation en deux pas de valeur 'pas'/2 de i à i+1/2 puis de i+1/2 à i+1
*
* La méthode renvoie le pas de temps adapté, qui peut ensuite être utiliser dans la méthode rk4.
*/


public static double rk4_pasVariable(Fonction f, double y0, double pas, double tol) throws SimuException {

    {
      double Ya, Yb, err = 1.0;

      while(Math.abs(err)> tol){
        try{
           Ya = Bibliotheque.rk4(f,y0,pas);   //Estimation apres 1 pas h
        } catch (Exception erreur){
           System.err.println (erreur.getMessage());
           throw new SimuException("Erreur interne");
        }

        try{
           Yb = Bibliotheque.rk4(f,y0,pas/2.0); //Estimation apres 1 pas h/2
        } catch (Exception erreur){
           System.err.println (erreur.getMessage());
           throw new SimuException("Erreur interne");
        }

        try{
           Yb = Bibliotheque.rk4(f,Yb,pas/2.0);     //Estimation apres 2 pas h/2
        } catch (Exception erreur){
           System.err.println (erreur.getMessage());
           throw new SimuException("Erreur interne");
        }

        if(Ya!=Yb){
           err=Math.abs(Yb-Ya) / ( Math.abs(Yb) + pas*Math.abs(f.y(Yb))) ;
           pas=0.95*pas*Math.pow(err/tol,-1./5.);
        }
        else err=0.0;
      }

      return pas;
    }
}


/**
 *
 * Runge-Kutta d'ordre 4 pour une équation différentielle d'ordre 2.
 * Pour une équation différentielle du type : Y'' = F(Y), où Y=(x,v), alors au pas i+1 on a :
 * Y(i+1)=Y(i)+h*1/6*(F1 + 2F2 + 2F3 + F4)
 *
 * où les Fi et Yi sont des vecteurs à deux composantes (position,vitesse), qui estiment F en différents points:
 * F1(Yi)
 * F2(Yi+h/2 F1)
 * F3(Yi+h/2 F2)
 * F4(Yi+h F3)
 *
 *
 */


    public static double[] rk4_2D(Fonction2D f, double x[], double pas)

    {
        double[] F1 = new double[2];
        double[] F2 = new double[2];
        double[] F3 = new double[2];
        double[] F4 = new double[2];
        double[] out= new double[2];

        F1=f.z(x[0],x[1]);
        F2=f.z(x[0]+pas*0.5*F1[0],x[1]+pas*0.5*F1[1]);
        F3=f.z(x[0]+pas*0.5*F2[0],x[1]+pas*0.5*F2[1]);
        F4=f.z(x[0]+pas*F3[0],x[1]+pas*F3[1]);

        out[0]=x[0] + pas * ( F1[0] + 2.*F2[0] + 2.*F3[0] + F4[0] ) /6.0;
        out[1]=x[1] + pas * ( F1[1] + 2.*F2[1] + 2.*F3[1] + F4[1] ) /6.0;

        return out;
    }

};




